# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

from typing import NamedTuple


class RegisterLabware_Responses(NamedTuple):

    ItemIdentifier: str
    """
    UUID identifier for referenceable labware, modules,
                adapters, etc.
    """


class DeleteLabware_Responses(NamedTuple):

    pass


class MoveLabware_Responses(NamedTuple):

    pass
