# Generated by sila2.code_generator; sila2.__version__: 0.12.2
from __future__ import annotations

from typing import NamedTuple


class Deactivate_Responses(NamedTuple):

    pass


class DeactivateBlockHeater_Responses(NamedTuple):

    pass


class DeactivateLidHeater_Responses(NamedTuple):

    pass


class CloseLid_Responses(NamedTuple):

    pass


class OpenLid_Responses(NamedTuple):

    pass


class SetLidTemperature_Responses(NamedTuple):

    pass


class SetBlockTemperature_Responses(NamedTuple):

    pass


class ExecuteProfile_Responses(NamedTuple):

    pass


class CurrentBlockTemperature_Responses(NamedTuple):

    Temperature: float
    """
    Temperature in Celsius
    """


class CurrentLidTemperature_Responses(NamedTuple):

    Temperature: float
    """
    Temperature in Celsius
    """


class CurrentLidTemperatureStatus_Responses(NamedTuple):

    TemperatureStatus: str
    """
    The temperature status
    """


class CurrentBlockTemperatureStatus_Responses(NamedTuple):

    TemperatureStatus: str
    """
    The temperature status
    """


class CurrentLidPosition_Responses(NamedTuple):

    LidPosition: str
    """
    The lid position
    """
